#
# Stuff for dealing with configuration files.
#
import sys
import database

#
# Read a line and strip out junk.
#
def ReadConfigLine (file):
    line = file.readline ()
    if not line:
        return None
    line = line.split('#')[0] # Get rid of any comments
    line = line.strip () # and extra white space
    if len (line) == 0: # we got rid of everything
        return ReadConfigLine (file)
    return line

#
# Give up and die.
#
def croak (message):
    sys.stderr.write (message + '\n')
    sys.exit (1)

#
# Read a list of email aliases.
#
def ReadEmailAliases (name):
    try:
        file = open (name, 'r')
    except IOError:
        croak ('Unable to open email alias file %s' % (name))
    line = ReadConfigLine (file)
    while line:
        sline = line.split ()
        if len (sline) != 2:
            croak ('Funky email alias line "%s"' % (line))
        if sline[0].index ('@') <= 0 or sline[1].index ('@') <= 0:
            croak ('Non-addresses in email alias "%s"' % (line))
        database.AddEmailAlias (sline[0], sline[1])
        line = ReadConfigLine (file)
    file.close ()

#
# The Email/Employer map
#
# Eventually we want to add date-based mappings here.
#
def ReadEmailEmployers (name):
    try:
        file = open (name, 'r')
    except IOError:
        croak ('Unable to open email/employer file %s' % (name))
    line = ReadConfigLine (file)
    while line:
        sline = line.split ()
        if len (sline) < 2:
            croak ('Funky email/employer line "%s"' % (line))
        database.AddEmailEmployerMapping (sline[0], ' '.join (sline[1:]))
        line = ReadConfigLine (file)
    file.close ()

def ReadGroupMap (fname, employer):
    try:
        file = open (fname, 'r')
    except IOError:
        croak ('Unable to open group map file %s' % (fname))
    line = ReadConfigLine (file)
    while line:
        database.AddEmailEmployerMapping (line, employer)
        line = ReadConfigLine (file)
    file.close ()

#
# Read an overall config file.
#
def ConfigFile (name):
    try:
        file = open (name, 'r')
    except IOError:
        croak ('Unable to open config file %s' % (name))
    line = ReadConfigLine (file)
    while line:
        sline = line.split ()
        if len (sline) < 2:
            croak ('Funky config line: "%s"' % (line))
        if sline[0] == 'EmailAliases':
            ReadEmailAliases (sline[1])
        elif sline[0] == 'EmailMap':
            ReadEmailEmployers (sline[1])
        elif sline[0] == 'GroupMap':
            if len (sline) != 3:
                croak ('Funky group map line "%s"' % (line))
            ReadGroupMap (sline[1], sline[2])
        else:
            croak ('Unrecognized config line: "%s"' % (line))
        line = ReadConfigLine (file)
        
