#
# The "database".
#

#
# This code is part of the LWN git data miner.
#
# Copyright 2007 LWN.net
# Copyright 2007 Jonathan Corbet <corbet@lwn.net>
#
# This file may be distributed under the terms of the GNU General
# Public License, version 2.
import sys


class Hacker:
    def __init__ (self, name, id, empl, email):
        self.name = name
        self.id = id
        self.employer = [ empl ]
        self.email = [ email ]
        self.added = self.removed = 0
        self.patches = [ ]
        self.signoffs = [ ]

    def addemail (self, email, empl):
        self.email.append (email)
        self.employer.append (empl)
        HackersByEmail[email] = self

    def emailemployer (self, email):
        for i in range (0, len (self.email)):
            if self.email[i] == email:
                return self.employer[i]
        print 'OOPS.  ', self.name, self.employer, self.email, email
        return None # Should not happen

    def addpatch (self, patch):
        self.added += patch.added
        self.removed += patch.removed
        self.patches.append (patch)

    def addsob (self, patch):
        self.signoffs.append (patch)

HackersByName = { }
HackersByEmail = { }
HackersByID = { }
MaxID = 0

def StoreHacker (name, empl, email):
    global MaxID

    id = MaxID
    MaxID += 1
    h = Hacker (name, id, empl, email)
    HackersByName[name] = h
    HackersByEmail[email] = h
    HackersByID[id] = h
    return h

def LookupEmail (addr):
    try:
        return HackersByEmail[addr]
    except KeyError:
        return None

def LookupName (name):
    try:
        return HackersByName[name]
    except KeyError:
        return None

def LookupID (id):
    try:
        return HackersByID[id]
    except KeyError:
        return None

def AllHackers ():
    return HackersByID.values ()
#    return [h for h in HackersByID.values ()] #  if (h.added + h.removed) > 0]

def DumpDB ():
    out = open ('database.dump', 'w')
    names = HackersByName.keys ()
    names.sort ()
    for name in names:
        h = HackersByName[name]
        out.write ('%4d %s %d p (+%d -%d) sob: %d\n' % (h.id, h.name,
                                                        len (h.patches),
                                                        h.added, h.removed,
                                                        len (h.signoffs)))
        for i in range (0, len (h.email)):
            out.write ('\t%s -> %s\n' % (h.email[i], h.employer[i].name))

#
# Employer info.
#
class Employer:
    def __init__ (self, name):
        self.name = name
        self.added = self.removed = self.count = self.changed = 0
        self.sobs = 0
        self.hackers = [ ]

    def AddCSet (self, patch):
        self.added += patch.added
        self.removed += patch.removed
        self.changed += max(patch.added, patch.removed)
        self.count += 1
        if patch.author not in self.hackers:
            self.hackers.append (patch.author)

    def AddSOB (self):
        self.sobs += 1

Employers = { }

def GetEmployer (name):
    try:
        return Employers[name]
    except KeyError:
        e = Employer (name)
        Employers[name] = e
        return e

def AllEmployers ():
    return Employers.values ()

#
# The email map.
#
EmailAliases = { }

def AddEmailAlias (variant, canonical):
    if EmailAliases.has_key (variant):
        sys.stderr.write ('Duplicate email alias for %s\n' % (variant))
    EmailAliases[variant] = canonical

def RemapEmail (email):
    email = email.lower ()
    try:
        return EmailAliases[email]
    except KeyError:
        return email

#
# Email-to-employer mapping.
#
EmailToEmployer = { }

def AddEmailEmployerMapping (email, employer):
    email = email.lower ()
    if EmailToEmployer.has_key (email):
        sys.stderr.write ('Duplicate email/employer for %s\n' % (email))
    EmailToEmployer[email] = employer

def MapToEmployer (email, unknown = 0):
    email = email.lower ()
    try:
        return EmailToEmployer[email]
    except KeyError:
        pass
    namedom = email.split ('@')
    if len (namedom) < 2:
        print 'Oops...funky email %s' % email
        return 'Funky'
    s = namedom[1].split ('.')
    for dots in range (len (s) - 2, -1, -1):
        addr = '.'.join (s[dots:])
        try:
            return EmailToEmployer[addr]
        except KeyError:
            pass
    if unknown:
        return '(Unknown)'
    return email


def LookupEmployer (email, mapunknown = 0):
    ename = MapToEmployer (email, mapunknown)
    return GetEmployer (ename)
